from typing import Any, Dict, List, Optional

import numpy as np
from sklearn.feature_extraction.text import CountVectorizer

from nlp_automl.vectorizer.base import Vectorizer


class CountVec(Vectorizer):
    """Simple wrap up on count vectorizer."""

    def __init__(
        self, trial: Optional[Any] = None, hyperparams: Optional[Dict[str, Any]] = None
    ):
        super().__init__(hyperparams)
        self.vect = (
            self.optuna_config(trial) if trial else self.default_config(hyperparams)
        )
        self.fitted = False

    def _vectorizer(self, hyperparams: Dict[str, Any]) -> Any:
        return CountVectorizer(
            max_features=hyperparams["max_features"],
            min_df=hyperparams["min_df"],
        )

    def default_config(self, hyperparams: Optional[Dict[str, Any]] = None) -> Any:
        hyperparams = hyperparams or {
            "max_features": 50000,
            "min_df": 0.001,
        }
        return self._vectorizer(hyperparams)

    def optuna_config(self, trial) -> Any:
        hyperparams = {
            "max_features": trial.suggest_int("max_features", 10, 1e5),
            "min_df": trial.suggest_float("min_df", 0.0001, 0.1, log=True),
        }
        return self._vectorizer(hyperparams)

    def fit(self, messages: List[str]) -> None:
        try:
            self.vect.fit(messages)
            self.fitted = True
        except ValueError:  # appears if min_df too high
            pass

    def transform_one(self, message: str) -> np.ndarray:
        if not self.fitted:
            return np.zeros((1, 1))
        return self.vect.transform([message]).toarray()[0]

    def transform(self, messages: List[str]) -> np.ndarray:
        if not self.fitted:
            return np.zeros((len(messages), 1))
        return self.vect.transform(messages).toarray()



from abc import abstractmethod
from typing import List

import numpy as np

from nlp_automl.base import Base


class Vectorizer(Base):
    """Base class for string vrctorizers."""

    @abstractmethod
    def fit(self, messages: List[str]) -> None:
        """It should do exactly how its named."""

    @abstractmethod
    def transform_one(self, message: str) -> np.ndarray:
        """Should vectorize message."""

    @abstractmethod
    def transform(self, messages: List[str]) -> np.ndarray:
        """Should vectorize list of messages."""

    def fit_transform(self, messages: List[str]) -> np.ndarray:
        """Should fit self and return transformed."""
        self.fit(messages)
        return self.transform(messages)



from abc import abstractmethod
from typing import List

from nlp_automl.base import Base


class Preprocessor(Base):
    """Base class for string preprocessing."""

    @abstractmethod
    def preprocess_one(self, message: str) -> str:
        """Should preprocess message according to config."""

    @abstractmethod
    def preprocess(self, messages: List[str]) -> List[str]:
        """Should preprocess message according to config."""



from typing import Any, Callable, Dict, List, Optional

import numpy as np

from nlp_automl.vectorizer.base import Vectorizer


def flatten(vec, embedding_size: int = 300):
    vec = np.array(vec)
    if vec.shape == (0,):
        return np.zeros((4 * embedding_size + 1,))
    arr = np.append(vec.mean(axis=0), vec.max(axis=0))
    arr = np.append(arr, vec.min(axis=0))
    arr = np.append(arr, vec.std(axis=0))
    return np.append(vec.shape[0], arr)


class FastText(Vectorizer):
    """Wrap up on 2D fast text embedding to make it 1D."""

    def __init__(
        self,
        trial: Optional[Any] = None,
        hyperparams: Optional[Dict[str, Any]] = None,
    ):
        super().__init__(hyperparams)
        if not hyperparams:
            raise ValueError('please provide "token_embedder" in hyperparams')
        self.token_embedder: Callable = hyperparams["token_embedder"]
        self.embedding_size: int = self.token_embedder("the").shape[0]

    def default_config(self, hyperparams: Optional[Dict[str, Any]] = None) -> Any:
        pass

    def optuna_config(self, trial) -> Any:
        pass

    def fit(self, messages: List[str]) -> None:
        pass

    def transform_one(self, message: str) -> np.ndarray:
        return flatten(
            [self.token_embedder(token) for token in message.split()],
            self.embedding_size,
        )

    def transform(self, messages: List[str]) -> np.ndarray:
        return np.array([self.transform_one(message) for message in messages])



from string import punctuation
from typing import Any, Dict, List, Optional, Set, Tuple

from nltk.corpus import stopwords
from pymystem3 import Mystem

from nlp_automl.preprocess.base import Preprocessor


class Lemmatizer(Preprocessor):
    """Makes token lemmas."""

    def __init__(
        self, trial: Optional[Any] = None, hyperparams: Optional[Dict[str, Any]] = None
    ):
        super().__init__(trial, hyperparams)
        self.mystem, self.stopwords = (
            self.optuna_config(trial) if trial else self.default_config(hyperparams)
        )

    def default_config(
        self, hyperparams: Optional[Dict[str, Any]] = None
    ) -> Tuple[Mystem, Set[str]]:
        hyperparams = hyperparams or {
            "delete_stopwords": True,
        }
        return Mystem(), set(stopwords.words("russian"))

    def optuna_config(self, trial) -> Tuple[Mystem, Set[str]]:
        hyperparams = {
            "delete_stopwords": trial.suggest_categorical(
                "delete_stopwords", [True, False]
            )
        }
        return self.default_config(hyperparams)

    def preprocess_one(self, message: str) -> str:
        message = message.lower().strip()
        tokens = [
            token
            for token in message.split()
            if token not in self.stopwords and token.strip() not in punctuation
        ]
        tokens = self.mystem.lemmatize(message.lower())
        return " ".join(tokens)

    def preprocess(self, messages: List[str]) -> List[str]:
        return [self.preprocess_one(msg) for msg in messages]



from typing import Any, Dict, Optional

import numpy as np
from sklearn.linear_model import LogisticRegression

from nlp_automl.method.base import MlMethod


class LogReg(MlMethod):
    """Simple wrap up on Log Reg linear model."""

    def __init__(
        self, trial: Optional[Any] = None, hyperparams: Optional[Dict[str, Any]] = None
    ):
        super().__init__(trial, hyperparams)
        self.seed = 777
        self.logreg = (
            self.optuna_config(trial) if trial else self.default_config(hyperparams)
        )

    def _logreg(self, hyperparams: Dict[str, Any]) -> LogisticRegression:
        return LogisticRegression(
            penalty="l2",  # {‘l1’, ‘l2’, ‘elasticnet’, ‘none’}, default=’l2’
            C=hyperparams["reg_strength"],  # Inverse of reg. strngth; float>0.
            fit_intercept=True,  # Specifies if a constant (a.k.a. bias or intercept)
            class_weight="balanced",  # The “balanced” to automatically adjust weights
            random_state=self.seed,  # RandomState instance, default=None
            solver=hyperparams["solver"],
            max_iter=hyperparams["max_iter"],
            multi_class="ovr",
            verbose=0,
            warm_start=hyperparams["warm_start"],
            n_jobs=1,  # Number of CPU cores used when parallelizing over classes
        )

    def default_config(
        self, hyperparams: Optional[Dict[str, Any]] = None
    ) -> LogisticRegression:
        hyperparams = {
            "reg_strength": 1.0,
            "solver": "lbfgs",
            "max_iter": 100,
            "warm_start": False,
        }
        return self._logreg(hyperparams)

    def optuna_config(self, trial) -> LogisticRegression:
        hyperparams = {
            "reg_strength": trial.suggest_float("reg_strength", 0.001, 5, log=True),
            "solver": trial.suggest_categorical("solver", ["lbfgs"]),
            "max_iter": trial.suggest_int("max_iter", 10, 300),
            "warm_start": trial.suggest_categorical("warm_start", [False]),
        }
        return self._logreg(hyperparams)

    def fit(self, features: np.ndarray, targets: np.ndarray) -> None:
        self.logreg.fit(features, targets)

    def predict(self, features: np.ndarray) -> np.ndarray:
        return self.logreg.predict(features)

    def predict_one(self, features: np.ndarray) -> np.ndarray:
        return self.logreg.predict([features])[0]



import re
from typing import Any, Dict, List, Optional

from nlp_automl.preprocess.base import Preprocessor


class Basic(Preprocessor):
    """Makes tokens."""

    def __init__(
        self, trial: Optional[Any] = None, hyperparams: Optional[Dict[str, Any]] = None
    ):
        super().__init__(trial, hyperparams)
        self.re_w, self.re_yo = (
            self.optuna_config(trial) if trial else self.default_config(hyperparams)
        )

    def default_config(self, hyperparams: Optional[Dict[str, Any]] = None) -> Any:
        return re.compile(r"[A-z,А-я]+"), re.compile(r"ё")

    def optuna_config(self, trial) -> Dict[str, Any]:
        return self.default_config()

    def preprocess_one(self, message: str) -> str:
        """Should preprocess message according to config."""
        message = message.lower()
        message = self.re_yo.sub("е", message)
        return " ".join(self.re_w.findall(message))

    def preprocess(self, messages: List[str]) -> List[str]:
        """Should preprocess message according to config."""
        return [self.preprocess_one(msg) for msg in messages]



from typing import Any, Dict, Optional

import numpy as np
from sklearn.ensemble import GradientBoostingClassifier

from nlp_automl.method.base import MlMethod


class Boosting(MlMethod):
    """Simple wrap up on Log Reg linear model."""

    def __init__(
        self, trial: Optional[Any] = None, hyperparams: Optional[Dict[str, Any]] = None
    ):
        super().__init__(trial, hyperparams)
        self.seed = 777
        self.boosting = (
            self.optuna_config(trial) if trial else self.default_config(hyperparams)
        )

    def _boosting(self, hyperparams: Dict[str, Any]) -> GradientBoostingClassifier:
        return GradientBoostingClassifier(
            n_estimators=hyperparams["n_estimators"],
            learning_rate=hyperparams["learning_rate"],
            max_depth=hyperparams["max_depth"],
            random_state=self.seed,
        )

    def default_config(
        self, hyperparams: Optional[Dict[str, Any]] = None
    ) -> GradientBoostingClassifier:
        hyperparams = {
            "n_estimators": 100,
            "learning_rate": 0.1,
            "max_depth": 3,
        }
        return self._boosting(hyperparams)

    def optuna_config(self, trial) -> GradientBoostingClassifier:
        hyperparams = {
            "n_estimators": trial.suggest_int("n_estimators", 10, 3000),
            "learning_rate": trial.suggest_float("learning_rate", 0.001, 5.0, log=True),
            "max_depth": trial.suggest_int("max_depth", 1, 3),
        }
        return self._boosting(hyperparams)

    def fit(self, features: np.ndarray, targets: np.ndarray) -> None:
        self.boosting.fit(features, targets)

    def predict(self, features: np.ndarray) -> np.ndarray:
        return self.boosting.predict(features)

    def predict_one(self, features: np.ndarray) -> np.ndarray:
        return self.boosting.predict([features])[0]



from abc import abstractmethod

import numpy as np

from nlp_automl.base import Base


class MlMethod(Base):
    """Base class for string vrctorizers."""

    @abstractmethod
    def fit(self, features: np.ndarray, targets: np.ndarray) -> None:
        """Should fit model."""

    @abstractmethod
    def predict(self, features: np.ndarray) -> np.ndarray:
        """Should predict on batch."""

    @abstractmethod
    def predict_one(self, features: np.ndarray) -> np.ndarray:
        """Should predict on one example."""



from typing import Any, Dict, Optional

import numpy as np
from sklearn.ensemble import RandomForestClassifier

from nlp_automl.method.base import MlMethod


class RandomForest(MlMethod):
    """Simple wrap up on Log Reg linear model."""

    def __init__(
        self, trial: Optional[Any] = None, hyperparams: Optional[Dict[str, Any]] = None
    ):
        super().__init__(trial, hyperparams)
        self.seed = 777
        self.rand_forest = (
            self.optuna_config(trial) if trial else self.default_config(hyperparams)
        )

    def _rand_forest(self, hyperparams: Dict[str, Any]) -> RandomForestClassifier:
        return RandomForestClassifier(
            n_estimators=hyperparams["n_estimators"], random_state=self.seed
        )

    def default_config(
        self, hyperparams: Optional[Dict[str, Any]] = None
    ) -> RandomForestClassifier:
        hyperparams = {
            "n_estimators": 100,
        }
        return self._rand_forest(hyperparams)

    def optuna_config(self, trial) -> RandomForestClassifier:
        hyperparams = {
            "n_estimators": trial.suggest_int(
                "n_estimators",
                10,
                3000,
            ),
        }
        return self._rand_forest(hyperparams)

    def fit(self, features: np.ndarray, targets: np.ndarray) -> None:
        self.rand_forest.fit(features, targets)

    def predict(self, features: np.ndarray) -> np.ndarray:
        return self.rand_forest.predict(features)

    def predict_one(self, features: np.ndarray) -> np.ndarray:
        return self.rand_forest.predict([features])[0]



from typing import Any, Dict

from sklearn.feature_extraction.text import TfidfVectorizer

from nlp_automl.vectorizer.count import CountVec


class TfIdfVectorizer(CountVec):
    """Simple wrap up on tf-idf vectorizer."""

    def _vectorizer(self, hyperparams: Dict[str, Any]) -> TfidfVectorizer:
        return TfidfVectorizer(
            max_features=hyperparams["max_features"],
            min_df=hyperparams["min_df"],
        )
